/*
 * connection.c - connection stuff for libsmtpc.
 *
 * (C) 2006 Carlos Eduardo Medaglia Dyonisio <cadu(a)nerdfeliz.com>
 *          http://nerdfeliz.com/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version
 * 2.1 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <string.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <poll.h>

#include <glib.h>

int connection_new(const char *host_name, const int port)
{
	int smtp_sock;
	struct hostent *smtp_host;
	struct sockaddr_in smtp_addr;

	smtp_sock = socket(PF_INET, SOCK_STREAM, 0);
	if(smtp_sock < 0)
		return -1;

	smtp_host = gethostbyname(host_name);

	smtp_addr.sin_family = AF_INET;
	smtp_addr.sin_addr = *(struct in_addr *)smtp_host->h_addr;
	smtp_addr.sin_port = htons(port);

	connect(smtp_sock, (struct sockaddr *)&smtp_addr, sizeof(smtp_addr));

	return smtp_sock;
}

char *connection_read(const int smtp_sock)
{
	char buf[2048];
	char *msg;
	int rb;
	struct pollfd fds;
	int ret;

	fds.fd = smtp_sock;
	fds.events = POLLIN | POLLPRI;
	
	ret = poll(&fds, 1, 120000);
	if(!ret)
		return NULL;

	if(fds.revents & POLLIN || fds.revents & POLLPRI) {
		// XXX: Should still be changed... (2048 bytes)
		rb = recv(smtp_sock, buf, 2048, 0);
		msg = malloc(rb+1);
		memcpy(msg, buf, rb);
		msg[rb] = '\0';

		return msg;
	}
	return NULL;
}

int connection_write(const int smtp_sock, const char *data)
{
	unsigned int data_size;
	int wb, ret;

	data_size = strlen(data);

	wb = 0;
	while(wb < data_size) {
		ret = write(smtp_sock, data+wb, data_size-wb);
		if(ret < 0)
			return ret;
		wb += ret;
	}
	return wb;
}
